{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Create Share Network" %}{% endblock %}

{% block main %}
    {% include 'project/shares/share_networks/_create_share_network.html' %}
{% endblock %}
