{% load i18n sizeformat parse_date %}
{% load url from future %}

<h3>{% trans "Snapshot Overview" %}</h3>
<div class="detail">
  <hr class="header_rule">
  <dl class="dl-horizontal">
    <dt>{% trans "Name" %}</dt>
    <dd>{{ snapshot.name }}</dd>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ snapshot.id }}</dd>
    {% url 'horizon:admin:shares:detail' snapshot.share_id as share_url %}
    <dt>{% trans "Source" %}</dt>
    <dd><a href="{{ share_url }}">{{ snapshot.share_name_or_id }}</a></dd>
    {% if snapshot.description %}
    <dt>{% trans "Description" %}</dt>
    <dd>{{ snapshot.description }}</dd>
    {% endif %}
    <dt>{% trans "Status" %}</dt>
    <dd>{{ snapshot.status|capfirst }}</dd>
  </dl>
</div>

<div class="detail">
  <h4>{% trans "Specs" %}</h4>
  <hr class="header_rule">
  <dl class="dl-horizontal">
    <dt>{% trans "Size" %}</dt>
    <dd>{{ snapshot.size }} {% trans "GiB" %}</dd>
    <dt>{% trans "Created" %}</dt>
    <dd>{{ snapshot.created_at|parse_date }}</dd>
  </dl>
</div>
