{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% load url from future %}

{% block form_id %}{% endblock %}
{% block form_action %}{% url 'horizon:project:shares:create_security_service' %}{% endblock %}

{% block modal_id %}create_security_service_modal{% endblock %}
{% block modal-header %}{% trans "Create Security Service" %}{% endblock %}

{% block modal-body %}
<div class="left">
  <fieldset>
  {% include "horizon/common/_form_fields.html" %}
  </fieldset>
</div>
<div class="right">
  <h3>{% trans "Description" %}:</h3>
  <p>{% blocktrans %}
    The security service can be used by backend drivers to 
    configure clients, for more secure using of share.
    {% endblocktrans %}</p>
</div>

{% endblock %}

{% block modal-footer %}
  <input class="btn btn-primary pull-right" type="submit" value="{% trans "Create Security Service" %}" />
  <a href="{% url 'horizon:project:shares:index' %}" class="btn secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
